from django.test import TestCase
from django.utils.unittest import skip
from django.contrib.auth.models import User

from iprestrict import models
from iprestrict import restrictor

class ReloadViewTest(TestCase):
    IP = '192.168.1.1'

    def setUp(self):
        admin = User.objects.create_user('admin', 'admin@nohost.org', 'pass')
        admin.is_staff = True
        admin.is_superuser = True
        admin.save()
        models.ReloadRulesRequest.request_reload()

    def add_allow_rule(self):
        localip = models.IPGroup.objects.create(name='Local IP')
        models.IPRange.objects.create(ip_group=localip, first_ip=self.IP)
        models.Rule.objects.create(url_pattern='ALL', ip_group = localip, action='A')

    def test_reload_view(self):
        # This test doesn't conform to unit test best practices.
        # Unfortunately, it does depend on order so it is in one method
        # instead of 3 methods, each asserting one thing

        # 1
        response = self.client.get('', REMOTE_ADDR = self.IP)
        self.assertEqual(response.status_code, 403, 'Should be restricted')

        # 2
        self.add_allow_rule()
        response = self.client.get('', REMOTE_ADDR = self.IP)
        self.assertEqual(response.status_code, 403, 'Should still be restricted - rules have not been reloaded')

        # 3 reload rules
        # TODO I have to log in as superuser for this to work
        self.client.login(username='admin', password='pass')
        response = self.client.get('/iprestrict/reload_rules')
        response = self.client.get('', REMOTE_ADDR = self.IP)
        self.assertEqual(response.status_code, 404, 'Should be allowed now')



from django.core.management.base import BaseCommand
from django.core.management import call_command

from iprestrict import models

class Command(BaseCommand):
    help = 'Replaces the current rules in the DB with the rules in the given fixture file(s).'
    args = "fixture [fixture ...]"

    def handle(self, *args, **options):
        verbosity = int(options.get('verbosity', '1'))

        self.delete_existing_rules()
        if verbosity >= 1:
            self.stdout.write('Successfully deleted rules\n')
       
        call_command('loaddata', *args,
            verbosity=verbosity,
            interactive=False)        

    def delete_existing_rules(self):
        models.Rule.objects.all().delete()
        models.IPRange.objects.all().delete()
        models.IPGroup.objects.all().delete()




from django.conf.urls import patterns, include, url

urlpatterns = patterns('iprestrict.views',
    url(r'^$', 'test_rules_page'),
    url(r'^move_rule_up/(?P<rule_id>\d+)[/]?$', 'move_rule_up'),
    url(r'^move_rule_down/(?P<rule_id>\d+)[/]?$', 'move_rule_down'),
    url(r'^reload_rules[/]?$', 'reload_rules'),
    url(r'^test_match[/]?$', 'test_match'),

)




from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings

from django.core import exceptions

from iprestrict import models
from iprestrict import restrictor
from iprestrict.middleware import IPRestrictMiddleware

from datetime import datetime

LOCAL_IP = '192.168.1.1'
PROXY = '1.1.1.1'

class MiddlewareRestrictsTest(TestCase):
    '''
    When the middleware is enabled it should restrict all IPs(but localhost)/URLs by default.
    '''
    def setUp(self):
        models.ReloadRulesRequest.request_reload()

    def assert_url_is_restricted(self, url):
        response = self.client.get(url, REMOTE_ADDR = LOCAL_IP)
        self.assertEqual(response.status_code, 403)

    def assert_ip_is_restricted(self, ip):
        response = self.client.get('', REMOTE_ADDR = ip)
        self.assertEqual(response.status_code, 403)

    def test_middleware_restricts_every_url(self):
        self.assert_url_is_restricted('')
        self.assert_url_is_restricted('/every')
        self.assert_url_is_restricted('/url')
        self.assert_url_is_restricted('/is_restricted')
        self.assert_url_is_restricted('/every/url/is_restricted')

    def test_middleware_restricts_ips(self):
        self.assert_ip_is_restricted('192.168.1.1')
        self.assert_ip_is_restricted('10.10.10.1')
        self.assert_ip_is_restricted('169.254.0.1')

    def test_middleware_allows_localhost(self):
        response = self.client.get('/some/url', REMOTE_ADDR = '127.0.0.1')
        self.assertEqual(response.status_code, 404)


def create_ip_allow_rule(ip=LOCAL_IP):
    localip = models.IPGroup.objects.create(name='localip')
    models.IPRange.objects.create(ip_group=localip, first_ip=LOCAL_IP)
    models.Rule.objects.create(url_pattern='ALL', ip_group = localip, action='A')

class MiddlewareAllowsTest(TestCase):
    def setUp(self):
        create_ip_allow_rule()
        models.ReloadRulesRequest.request_reload()

    def test_middleware_allows_localhost(self):
        response = self.client.get('')
        self.assertEqual(response.status_code, 404)

    def test_middleware_allows_ip_just_added(self):
        response = self.client.get('', REMOTE_ADDR = LOCAL_IP)
        self.assertEqual(response.status_code, 404)

    def test_middleware_restricts_other_ip(self):
        response = self.client.get('', REMOTE_ADDR = '10.1.1.1')
        self.assertEqual(response.status_code, 403)

    @override_settings(TRUSTED_PROXIES=(PROXY,))
    def test_middleware_allows_if_proxy_is_trusted(self):
        response = self.client.get('', REMOTE_ADDR = PROXY, HTTP_X_FORWARDED_FOR= LOCAL_IP)
        self.assertEqual(response.status_code, 404)

    def test_middleware_restricts_if_proxy_is_not_trusted(self):
        response = self.client.get('', REMOTE_ADDR = PROXY, HTTP_X_FORWARDED_FOR = LOCAL_IP)
        self.assertEqual(response.status_code, 403)

class ReloadRulesTest(TestCase):
    def setUp(self):
        create_ip_allow_rule()

    def test_reload_with_custom_command(self):
        from django.core.management import call_command
        call_command('reloadrules', verbosity=0)

        response = self.client.get('', REMOTE_ADDR = LOCAL_IP)
        self.assertEqual(response.status_code, 404)

class MiddlewareExtractClientIpTest(TestCase):
    def setUp(self):
        self.middleware = IPRestrictMiddleware()
        self.factory = RequestFactory()
 
    def test_remote_addr_only(self):
        self.middleware = IPRestrictMiddleware()
        request = self.factory.get('', REMOTE_ADDR=LOCAL_IP)

        client_ip = self.middleware.extract_client_ip(request)
        self.assertEquals(client_ip, LOCAL_IP)

    @override_settings(TRUSTED_PROXIES=(PROXY,))
    def test_single_proxy(self):
        self.middleware = IPRestrictMiddleware()
        request = self.factory.get('', REMOTE_ADDR=PROXY, HTTP_X_FORWARDED_FOR = LOCAL_IP)

        client_ip = self.middleware.extract_client_ip(request)
        self.assertEquals(client_ip, LOCAL_IP)

    @override_settings(TRUSTED_PROXIES=(PROXY,'2.2.2.2','4.4.4.4'))
    def test_multiple_proxies_one_not_trusted(self):
        self.middleware = IPRestrictMiddleware()
        proxies = ['2.2.2.2', '3.3.3.3', '4.4.4.4']
        request = self.factory.get('', REMOTE_ADDR=PROXY, 
           HTTP_X_FORWARDED_FOR = ', '.join([LOCAL_IP] + proxies))
        
        try:
            client_ip = self.middleware.extract_client_ip(request)
        except exceptions.PermissionDenied:
            pass
        else:
            self.fail('Should raise PermissionDenied exception')

    @override_settings(TRUSTED_PROXIES=(PROXY,'2.2.2.2','3.3.3.3', '4.4.4.4'))
    def test_multiple_proxies_all_trusted(self):
        self.middleware = IPRestrictMiddleware()
        proxies = ['2.2.2.2', '3.3.3.3', '4.4.4.4']
        request = self.factory.get('', REMOTE_ADDR=PROXY, 
           HTTP_X_FORWARDED_FOR = ', '.join([LOCAL_IP] + proxies))
        
        client_ip = self.middleware.extract_client_ip(request)
        self.assertEquals(client_ip, LOCAL_IP)




from django.utils.unittest import TestCase, skip

from iprestrict import ip_utils as ipu
        
class IPToNumberTest(TestCase):

    def test_ip_to_number_conversions(self):
        self.assertEqual(0, ipu.to_number('0.0.0.0'))
        self.assertEqual(1, ipu.to_number('0.0.0.1'))
        self.assertEqual(256, ipu.to_number('0.0.1.0'))
        self.assertEqual(65537, ipu.to_number('0.1.0.1'))
        self.assertEqual(16842753, ipu.to_number('1.1.0.1'))

    @skip('IPv6 to number conversion not ready')
    def test_ip_to_number_conversions_ipv6(self):
        self.assertEqual(0, ipu_to_number('::'))

class NumberToIPTest(TestCase):

    def test_number_to_ip_conversions(self):
        self.assertEqual('0.0.0.0', ipu.to_ip(0))
        self.assertEqual('0.0.0.1', ipu.to_ip(1))
        self.assertEqual('0.0.1.0', ipu.to_ip(256))
        self.assertEqual('0.1.0.1', ipu.to_ip(65537))
        self.assertEqual('1.1.0.1', ipu.to_ip(16842753))
 



from django.core import exceptions
from django.utils import log
from iprestrict.models import ReloadRulesRequest

from iprestrict import IPRestrictor

from django.conf import settings

logger = log.getLogger(__name__)

class IPRestrictMiddleware(object):

    def __init__(self):
        self.restrictor = IPRestrictor()
        self.trusted_proxies = getattr(settings, 'TRUSTED_PROXIES', tuple())
        self.dont_reload_rules = getattr(settings, 'DONT_RELOAD_RULES', False)

    def process_request(self, request):
        if not self.dont_reload_rules:
            self.reload_rules_if_needed()

        url = request.path_info
        client_ip = self.extract_client_ip(request)

        if self.restrictor.is_restricted(url, client_ip):
            logger.info("Denying access of %s to %s" % (url, client_ip))
            raise exceptions.PermissionDenied

    def extract_client_ip(self, request):
        client_ip = request.META['REMOTE_ADDR']
        forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if forwarded_for is not None:
            forwarded_for = [ip.strip() for ip in forwarded_for.split(',')]
            closest_proxy = client_ip
            client_ip = forwarded_for.pop(0)
            proxies = [closest_proxy] + forwarded_for
            for proxy in proxies:
                if proxy not in self.trusted_proxies:
                    logger.info("Client IP %s forwarded by untrusted proxy %s"
                        % (client_ip, proxy))
                    raise exceptions.PermissionDenied 
 
        return client_ip

    def reload_rules_if_needed(self):
        last_reload_request = ReloadRulesRequest.last_request()
        if last_reload_request is not None:
            if self.restrictor.last_reload < last_reload_request:
                self.restrictor.reload_rules()




from iprestrict import models
from django.contrib import admin
from django import forms
import ip_utils as ipu

class RuleAdmin(admin.ModelAdmin):
    model = models.Rule

    exclude = ('rank',)
    list_display = ('url_pattern', 'ip_group', 'is_allowed', 'move_up_url', 'move_down_url')

class IPRangeForm(forms.ModelForm):
    class Meta:
        model = models.IPRange

    def clean_cidr_prefix_length(self):
        cidr = self.cleaned_data['cidr_prefix_length']
        if cidr:
            if not (1 <= cidr <= 31):
                raise forms.ValidationError("Must be a number between 1 and 31")
                
        return cidr

    def clean(self):
        cleaned_data = super(IPRangeForm, self).clean()
        first_ip = cleaned_data.get('first_ip')
        if first_ip is None:
            # first_ip is Mandatory, so just let the default validator catch this
            return cleaned_data
        last_ip = cleaned_data['last_ip']
        cidr = cleaned_data['cidr_prefix_length']

        if last_ip and cidr:
            raise forms.ValidationError("Don't specify the Last IP if you specified a CIDR prefix length")
        if last_ip:
            if ipu.get_version(first_ip) != ipu.get_version(last_ip):
                raise forms.ValidationError("Last IP should be the same type as First IP (%s)" % ipu.get_version(first_ip))
            if ipu.get_version(last_ip) != 'ipv6':
                # Ignore rest of validation for ipv6, support isn't there yet
                if ipu.to_number(first_ip) > ipu.to_number(last_ip):
                    raise forms.ValidationError("Last IP should be greater than First IP")
 
        if cidr:
            # With CIDR the starting address could be different than the one
            # the user specified. Making sure it is set to the first ip in the
            # subnet.
            start, end = ipu.cidr_to_range(first_ip, cidr)
            cleaned_data['first_ip'] = ipu.to_ip(start)

        return cleaned_data

class IPRangeInline(admin.TabularInline):
    model = models.IPRange
    form = IPRangeForm

    fields = ['first_ip', 'cidr_prefix_length', 'last_ip', 'ip_type']
    readonly_fields = ['ip_type']
    extra = 2

class IPGroupAdmin(admin.ModelAdmin):
    model = models.IPGroup
    inlines = [IPRangeInline]

admin.site.register(models.Rule, RuleAdmin)
admin.site.register(models.IPGroup, IPGroupAdmin)




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        # Default IP Groups: ALL and localhost
        all_group = orm.IPGroup.objects.create(
            name='ALL', description="Matches all IP Addresses")
        localhost_group = orm.IPGroup.objects.create(
            name='localhost', description="IP Address of localhost")
      
        # IP ranges defining ALL and localhost for ipv4 and ipv6  
        orm.IPRange.objects.create(ip_group=all_group,
            first_ip = '0.0.0.0', last_ip = '255.255.255.255')
        orm.IPRange.objects.create(ip_group=all_group,
            first_ip = '0:0:0:0:0:0:0:0', 
            last_ip = 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff')

        orm.IPRange.objects.create(ip_group=localhost_group,
            first_ip = '127.0.0.1')
        orm.IPRange.objects.create(ip_group=all_group,
            first_ip = '::1') 

        # Default rules: Allow all for localhost and Deny everything else

        orm.Rule.objects.create(ip_group=localhost_group,
            action = 'A', url_pattern = 'ALL', rank=65535) 
        orm.Rule.objects.create(ip_group=all_group,
            action = 'D', url_pattern = 'ALL', rank=65536) 


    def backwards(self, orm):
        raise RuntimeError("Cannot reverse this migration.")

    models = {
        'iprestrict.ipgroup': {
            'Meta': {'object_name': 'IPGroup'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'iprestrict.iprange': {
            'Meta': {'object_name': 'IPRange'},
            'cidr_prefix_length': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'first_ip': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['iprestrict.IPGroup']"}),
            'last_ip': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39', 'null': 'True', 'blank': 'True'})
        },
        'iprestrict.reloadrulesrequest': {
            'Meta': {'object_name': 'ReloadRulesRequest'},
            'at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'iprestrict.rule': {
            'Meta': {'ordering': "['rank', 'id']", 'object_name': 'Rule'},
            'action': ('django.db.models.fields.CharField', [], {'default': "'D'", 'max_length': '1'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_group': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': "orm['iprestrict.IPGroup']"}),
            'rank': ('django.db.models.fields.IntegerField', [], {'blank': 'True'}),
            'url_pattern': ('django.db.models.fields.CharField', [], {'max_length': '500'})
        }
    }

    complete_apps = ['iprestrict']



from django.core.management.base import BaseCommand
from iprestrict import models

class Command(BaseCommand):
    help = 'Requests the reload of the ip restriction rules from the DB.'

    def handle(self, *args, **options):
        verbosity = int(options.get('verbosity', '1'))
        
        models.ReloadRulesRequest.request_reload()
        if verbosity >= 1:
            self.stdout.write('Successfully requested reload of rules\n')



# This file is to be used for testing only

from django.conf.urls import patterns, include, url
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('iprestrict.views',
    url(r'^iprestrict/$', 'test_rules_page'),
    url(r'^iprestrict/move_rule_up/(?P<rule_id>\d+)[/]?$', 'move_rule_up'),
    url(r'^iprestrict/move_rule_down/(?P<rule_id>\d+)[/]?$', 'move_rule_down'),
    url(r'^iprestrict/reload_rules[/]?$', 'reload_rules'),
    url(r'^iprestrict/test_match[/]?$', 'test_match'),
)

urlpatterns += patterns('',
    url(r'^admin/', include(admin.site.urls)),
)

urlpatterns += staticfiles_urlpatterns()




#!/usr/bin/env python

from setuptools import setup

setup(
    name='django-iprestrict',
    version='0.3.2',
    description='Django app + middleware to restrict access to all or sections of a Django project by client IP ranges',
    long_description='Django app + middleware to restrict access to all or sections of a Django project by client IP ranges',
    author='Tamas Szabo, CCG, Murdoch University',
    author_email='devops@ccg.murdoch.edu.au',
    url='https://github.com/muccg/django-iprestrict',
    download_url='https://github.com/muccg/django-iprestrict/releases',
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development"
    ],
    packages=[
        'iprestrict',
        'iprestrict.management',
        'iprestrict.management.commands',
    ],
    include_package_data=True,
    install_requires=[
        'django-templatetag-handlebars==1.2.0',
    ]
)



from django.test import TestCase
from django.http import HttpResponseForbidden

import iprestrict
from iprestrict import models
        
class RuleTest(TestCase):
    def test_restriction_methods_for_allow_rule(self):
        rule = models.Rule(url_pattern='', action='A')
        self.assertTrue(rule.is_allowed())
        self.assertFalse(rule.is_restricted())

    def test_restiction_methods_for_deny_rule(self):
        rule = models.Rule(url_pattern='', action='D')
        self.assertFalse(rule.is_allowed())
        self.assertTrue(rule.is_restricted())

    def test_matches_url_pattern_regex(self):
        rule = models.Rule(url_pattern='^/pre/[a-d]+[/]?$')
        self.assertTrue(rule.matches_url('/pre/a/'))
        self.assertTrue(rule.matches_url('/pre/a'))
        self.assertFalse(rule.matches_url('/pre/e/'))
        self.assertFalse(rule.matches_url('/pre/a//'))

    def test_rank_is_automatically_assigned_on_creation(self):
        rule1 = models.Rule.objects.create(url_pattern='1', action='A')
        rule2 = models.Rule.objects.create(url_pattern='2', action='A')
        rule10 = models.Rule.objects.create(url_pattern='10', action='A', rank=10)
        rule11 = models.Rule.objects.create(url_pattern='10', action='A')
        self.assertEquals(rule1.rank, 1)
        self.assertEquals(rule2.rank, 2)
        self.assertEquals(rule10.rank, 10)
        self.assertEquals(rule11.rank, 11)

    def test_rank_is_not_changed_on_update(self):
        rule1 = models.Rule.objects.create(url_pattern='1', action='A')
        rule2 = models.Rule.objects.create(url_pattern='2', action='A')
        rule1.action = 'D'
        rule1.save()
        self.assertEquals(rule1.rank, 1)

class RuleWithSampleRulesTests(TestCase):
    def setUp(self):
        self.rule1 = models.Rule.objects.create(url_pattern='1', action='A')
        self.rule2 = models.Rule.objects.create(url_pattern='2', action='A')

    def test_default_order_is_by_rank(self):
        rules = models.Rule.objects.all()
        self.assertEqual(rules[0].url_pattern, '1')
        self.assertEqual(rules[1].url_pattern, '2')

    def test_move_up(self):
        self.rule2.move_up()
        rules = models.Rule.objects.all()
        self.assertEqual(rules[0].url_pattern, '2')
        self.assertEqual(rules[1].url_pattern, '1')

    def test_move_up_first_rule_does_nothing(self):
        self.rule1.move_up()
        rules = models.Rule.objects.all()
        self.assertEqual(rules[0].url_pattern, '1')
        self.assertEqual(rules[1].url_pattern, '2')

    def test_move_down(self):
        self.rule1.move_down()
        rules = models.Rule.objects.all()
        self.assertEqual(rules[0].url_pattern, '2')
        self.assertEqual(rules[1].url_pattern, '1')

    def test_move_down_below_default_rule(self):
        self.rule1.move_down()
        self.rule1.move_down()
        rules = models.Rule.objects.all()
        self.assertEqual(rules[0].url_pattern, '2')
        self.assertEqual(rules[1].url_pattern, 'ALL')
        self.assertEqual(rules[2].url_pattern, '1')


class IPGroupTest(TestCase):
    def test_first_ip_group_is_all(self):
        '''An IP definition matching all should be inserted by default'''
        all_group = models.IPGroup.objects.get(name='ALL')
        self.assertTrue(all_group.matches('192.168.1.1'))
        self.assertTrue(all_group.matches('200.200.200.200'))
        self.assertTrue(all_group.matches('1.2.3.4'))

    def test_matches_with_ranges(self):
        ipgroup = models.IPGroup.objects.create(name='Local IPs')
        iprange = models.IPRange.objects.create(ip_group=ipgroup, first_ip='192.168.1.1', last_ip='192.168.1.10')
        iprange2 = models.IPRange.objects.create(ip_group=ipgroup, first_ip='192.168.1.100', last_ip='192.168.1.110')

        ipgroup.load_ranges()

        self.assertTrue(ipgroup.matches('192.168.1.1'))
        self.assertTrue(ipgroup.matches('192.168.1.5'))
        self.assertTrue(ipgroup.matches('192.168.1.10'))
        self.assertTrue(ipgroup.matches('192.168.1.105'))
        self.assertTrue(ipgroup.matches('192.168.1.100'))
        self.assertFalse(ipgroup.matches('192.168.1.0'))
        self.assertFalse(ipgroup.matches('192.168.1.11'))
        self.assertFalse(ipgroup.matches('192.168.1.99'))

    def test_ipv6_and_ip4_are_separated(self):
        ipgroup = models.IPGroup.objects.create(name='Local IPs')
        iprange = models.IPRange.objects.create(ip_group=ipgroup, first_ip='::1')
        iprange = models.IPRange.objects.create(ip_group=ipgroup, first_ip='0.0.0.2')
        ipgroup.load_ranges()

        self.assertFalse(ipgroup.matches('0.0.0.1'))
        self.assertTrue(ipgroup.matches('0.0.0.2'))
        # TODO enable when IPv6 conversion to number works
        #self.assertFalse(ipgroup.matches('::2'))
        #self.assertTrue(ipgroup.matches('::1'))

    def test_matches_with_subnets(self):
        ipgroup = models.IPGroup.objects.create(name='Local IPs')
        iprange = models.IPRange.objects.create(ip_group=ipgroup, first_ip='192.168.1.0', cidr_prefix_length=30)

        ipgroup.load_ranges()

        self.assertTrue(ipgroup.matches('192.168.1.0'))
        self.assertTrue(ipgroup.matches('192.168.1.1'))
        self.assertTrue(ipgroup.matches('192.168.1.2'))
        self.assertTrue(ipgroup.matches('192.168.1.3'))
        self.assertFalse(ipgroup.matches('192.168.0.255'))
        self.assertFalse(ipgroup.matches('192.168.1.4'))

    def test_matches_subnet_first_ip_not_correct(self):
        ipgroup = models.IPGroup.objects.create(name='Local IPs')
        iprange = models.IPRange.objects.create(ip_group=ipgroup, 
            first_ip='192.168.1.2', # Should be '192.168.1.1'
            cidr_prefix_length=30)

        ipgroup.load_ranges()

        self.assertTrue(ipgroup.matches('192.168.1.0'))
        self.assertTrue(ipgroup.matches('192.168.1.1'))
        self.assertTrue(ipgroup.matches('192.168.1.2'))
        self.assertTrue(ipgroup.matches('192.168.1.3'))
        self.assertFalse(ipgroup.matches('192.168.0.255'))
        self.assertFalse(ipgroup.matches('192.168.1.4'))

       




from django.test import TestCase
from django.utils import unittest

import iprestrict
from iprestrict import models

IP = '10.1.1.1'
SOME_URL = '/some/url'
        
class IPRestrictorDefaultRulesTest(TestCase):

    def test_restrictor_restricts_all_by_default(self):
        restr = iprestrict.IPRestrictor()
        self.assertTrue(restr.is_restricted('', IP))
        self.assertTrue(restr.is_restricted(SOME_URL, IP))

    def test_allow_all_ips_for_one_url(self):
        models.Rule.objects.create(url_pattern=SOME_URL, action='A', rank=1)
        restr = iprestrict.IPRestrictor()
        self.assertFalse(restr.is_restricted(SOME_URL, IP))
        self.assertFalse(restr.is_restricted(SOME_URL, '192.168.1.1'))

    def test_caches_rules(self):
        restr = iprestrict.IPRestrictor()
        models.Rule.objects.create(url_pattern=SOME_URL, action='A', rank=1)
        # Restrictor shouldn't read the rules dynamically
        self.assertTrue(restr.is_restricted(SOME_URL, IP))

    def test_reload_rules(self):
        restr = iprestrict.IPRestrictor()
        models.Rule.objects.create(url_pattern=SOME_URL, action='A', rank=1)
        restr.reload_rules()
        self.assertFalse(restr.is_restricted(SOME_URL, IP))


class IPRestrictorNoRulesTest(TestCase):

    def setUp(self):
        models.Rule.objects.all().delete()
        self.restrictor = iprestrict.IPRestrictor()

    def test_restrictor_does_not_do_anything_with_empty_rules_table(self):
        self.assertFalse(self.restrictor.is_restricted('', IP))
        self.assertFalse(self.restrictor.is_restricted(SOME_URL, '192.168.1.1'))

    def test_restrictor_considers_rules_by_rank(self):
        rule = models.Rule.objects.create(url_pattern='ALL', action='A', rank=2)
        rule = models.Rule.objects.create(url_pattern='/admin[/].*',  action='D', rank=1)
        restr = iprestrict.IPRestrictor()
        self.assertFalse(restr.is_restricted('/some/url', IP))
        self.assertTrue(restr.is_restricted('/admin/', IP))
        self.assertTrue(restr.is_restricted('/admin/somepage', IP))


class IPRestrictorOneUrlAllowedFromOneIpTest(TestCase):
    def setUp(self):
        localip = models.IPGroup.objects.create(name='localip')
        models.IPRange.objects.create(ip_group=localip, first_ip=IP)
        models.Rule.objects.create(url_pattern=SOME_URL, ip_group=localip, action='A')
        self.restrictor = iprestrict.IPRestrictor()

    def test_ip_matches_url_does_not(self):
        self.assertTrue(self.restrictor.is_restricted('', IP))

    def test_url_matches_ip_does_not(self):
        self.assertTrue(self.restrictor.is_restricted(SOME_URL, '192.168.1.1'))

    def test_both_match(self):
        self.assertFalse(self.restrictor.is_restricted(SOME_URL, IP))

    def test_reload_if_ipgroup_changed(self):
        self.assertTrue(self.restrictor.is_restricted(SOME_URL, '10.10.10.10'))
        localhost = models.IPGroup.objects.get(name='localhost')
        models.IPRange.objects.create(ip_group=localhost, first_ip='10.10.10.10')

        self.assertTrue(self.restrictor.is_restricted(SOME_URL, '10.10.10.10'))
        self.restrictor.reload_rules()
        self.assertFalse(self.restrictor.is_restricted(SOME_URL, '10.10.10.10'))
        



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'IPGroup'
        db.create_table('iprestrict_ipgroup', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('iprestrict', ['IPGroup'])

        # Adding model 'IPRange'
        db.create_table('iprestrict_iprange', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('ip_group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['iprestrict.IPGroup'])),
            ('first_ip', self.gf('django.db.models.fields.GenericIPAddressField')(max_length=39)),
            ('cidr_prefix_length', self.gf('django.db.models.fields.PositiveSmallIntegerField')(null=True, blank=True)),
            ('last_ip', self.gf('django.db.models.fields.GenericIPAddressField')(max_length=39, null=True, blank=True)),
        ))
        db.send_create_signal('iprestrict', ['IPRange'])

        # Adding model 'Rule'
        db.create_table('iprestrict_rule', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('url_pattern', self.gf('django.db.models.fields.CharField')(max_length=500)),
            ('ip_group', self.gf('django.db.models.fields.related.ForeignKey')(default=1, to=orm['iprestrict.IPGroup'])),
            ('action', self.gf('django.db.models.fields.CharField')(default='D', max_length=1)),
            ('rank', self.gf('django.db.models.fields.IntegerField')(blank=True)),
        ))
        db.send_create_signal('iprestrict', ['Rule'])

        # Adding model 'ReloadRulesRequest'
        db.create_table('iprestrict_reloadrulesrequest', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('iprestrict', ['ReloadRulesRequest'])


    def backwards(self, orm):
        
        # Deleting model 'IPGroup'
        db.delete_table('iprestrict_ipgroup')

        # Deleting model 'IPRange'
        db.delete_table('iprestrict_iprange')

        # Deleting model 'Rule'
        db.delete_table('iprestrict_rule')

        # Deleting model 'ReloadRulesRequest'
        db.delete_table('iprestrict_reloadrulesrequest')


    models = {
        'iprestrict.ipgroup': {
            'Meta': {'object_name': 'IPGroup'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'iprestrict.iprange': {
            'Meta': {'object_name': 'IPRange'},
            'cidr_prefix_length': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'first_ip': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['iprestrict.IPGroup']"}),
            'last_ip': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39', 'null': 'True', 'blank': 'True'})
        },
        'iprestrict.reloadrulesrequest': {
            'Meta': {'object_name': 'ReloadRulesRequest'},
            'at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'iprestrict.rule': {
            'Meta': {'ordering': "['rank', 'id']", 'object_name': 'Rule'},
            'action': ('django.db.models.fields.CharField', [], {'default': "'D'", 'max_length': '1'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'ip_group': ('django.db.models.fields.related.ForeignKey', [], {'default': '1', 'to': "orm['iprestrict.IPGroup']"}),
            'rank': ('django.db.models.fields.IntegerField', [], {'blank': 'True'}),
            'url_pattern': ('django.db.models.fields.CharField', [], {'max_length': '500'})
        }
    }

    complete_apps = ['iprestrict']



from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from iprestrict import models
from iprestrict.restrictor import IPRestrictor
from iprestrict.decorators import superuser_required
from django.shortcuts import render_to_response

import json

@superuser_required
def move_rule_up(request, rule_id):
    rule = models.Rule.objects.get(pk=rule_id)
    rule.move_up()
    return HttpResponseRedirect(reverse('admin:iprestrict_rule_changelist'))

@superuser_required
def move_rule_down(request, rule_id):
    rule = models.Rule.objects.get(pk=rule_id)
    rule.move_down()
    return HttpResponseRedirect(reverse('admin:iprestrict_rule_changelist'))

@superuser_required
def reload_rules(request):
    models.ReloadRulesRequest.request_reload()
    return HttpResponse('ok')

@superuser_required
def test_rules_page(request):
    return render_to_response('iprestrict/test_rules.html')

@superuser_required
def test_match(request):
    url = request.REQUEST['url']
    ip = request.REQUEST['ip']

    matching_rule_id, action = find_matching_rule(url, ip)
    rules = list_rules(matching_rule_id, url, ip)

    if matching_rule_id is None:
        result = {
            'action': 'Allowed',
            'msg': 'No rules matched.',
        }
    else:
        result = {
            'action': action,
            'msg': 'URL matched Rule highlighted below.'
        }
    result['rules'] = rules

    return HttpResponse(json.dumps(result))

def find_matching_rule(url, ip):
    for r in models.Rule.objects.all():
        if r.matches_url(url) and r.matches_ip(ip):
            return (r.pk, r.action_str())
    return (None, None)

def list_rules(matching_rule_id, url, ip):
    return [map_rule(r, matching_rule_id, url, ip) for r in models.Rule.objects.all()]

def map_rule(r, matching_rule_id, url, ip):
    rule = {
        'url_pattern': {
            'value': r.url_pattern,
            'matchStatus': 'match' if r.matches_url(url) else 'noMatch'
        },
        'ip_group': {
            'name':  r.ip_group.name,
            'ranges': r.ip_group.ranges_str(),
            'matchStatus': 'match' if r.matches_ip(ip) else 'noMatch'
        },
        'action': r.action_str(),
    }
    if r.pk == matching_rule_id:
        rule['matched'] = True
    return rule




def get_version(ip):
    return 'ipv4' if '.' in ip else 'ipv6'

def to_number(ip):
    parts= ip.split('.')
    parts = [int(p) for p in reversed(parts)]
    nr = 0
    for i, d in enumerate(parts):
       nr += (256 ** i) * d
    return nr

def to_ip(number):
    mask = int('1' * 8, 2)
    parts = []
    for i in range(4):
        shifted_number = number >> (8 * i)
        parts.append(shifted_number & mask) 

    return '.'.join(map(lambda i: str(i), reversed(parts)))

def cidr_to_range(ip, prefix_length):
    ip = to_number(ip)
    start_mask = int('1' * prefix_length, 2) << (32 - prefix_length)
    end_mask = int('1' * (32 - prefix_length), 2)
    start = ip & start_mask
    end = start | end_mask
    return (start, end) 




from iprestrict import models
from datetime import datetime

class IPRestrictor(object):
    def __init__(self):
        self.load_rules()

    def is_restricted(self, url, ip):
        for rule in self.rules:
            if rule.matches_url(url) and rule.matches_ip(ip):
                return rule.is_restricted()
        return False

    def load_rules(self):
        # We are caching the rules, to avoid DB lookup on each request
        self.rules = [r for r in models.Rule.objects.all()]
        self.last_reload = datetime.now()

    reload_rules = load_rules
